import { SemanticMark } from "./semantic-mark";

import { GrammarMarkType, Factory, SIGNAL_VIEW_BOX } from "@visactor/vgrammar-core";

import { PlotMakType } from "./enums";

import { field as getFieldAccessor } from "@visactor/vgrammar-util";

export class TreemapSemanticMark extends SemanticMark {
    constructor(id) {
        super(PlotMakType.treemap, id), Factory.getTransform("treemap") || this._logger.error("Please add this line of code: import { registerTreemapTransforms } from 'vgrammar-hierarchy'; \n        and run registerTreemapTransforms() before use treemap");
    }
    setMarkType() {
        return GrammarMarkType.rect;
    }
    setDefaultDataTransform() {
        var _a;
        return [ {
            type: "treemap",
            width: {
                signal: "viewWidth"
            },
            height: {
                signal: "viewHeight"
            },
            nodeKey: null === (_a = this.spec.encode) || void 0 === _a ? void 0 : _a.node,
            flatten: !0
        } ];
    }
    parseScaleByEncode(channel, option) {
        return "color" === channel ? {
            type: "ordinal",
            id: this.getScaleId("color"),
            domain: {
                data: this.getDataIdOfFiltered(),
                field: option
            },
            range: this.getPalette()
        } : null;
    }
    convertMarkEncode(encode) {
        var _a, _b, _c;
        const markEncoder = this.convertSimpleMarkEncode(encode), res = {
            x: {
                field: "x0"
            },
            x1: {
                field: "x1"
            },
            y: {
                field: "y0"
            },
            y1: {
                field: "y1"
            }
        };
        if (markEncoder.stroke && (res.stroke = markEncoder.stroke), markEncoder.color) {
            const scaleColorId = this.getScaleId("color"), colorAccessor = getFieldAccessor(markEncoder.color.field);
            res.fill = (datum, el, params) => {
                const scale = params[scaleColorId];
                return (null == datum ? void 0 : datum.datum) ? scale.scale(colorAccessor(datum.datum[datum.datum.length - 1])) : void 0;
            };
        } else res.fill = null !== (_b = null === (_a = this.spec.style) || void 0 === _a ? void 0 : _a.fill) && void 0 !== _b ? _b : null === (_c = this.getPalette()) || void 0 === _c ? void 0 : _c[0];
        return res;
    }
    setMainMarkSpec() {
        return {
            key: "key"
        };
    }
    parseLabelSpec() {
        return {
            marks: [],
            interactions: []
        };
    }
    setMultiMarksSpec() {
        const label = this.spec.label;
        return label ? Object.keys(label).map((key => {
            const textGetter = getFieldAccessor(key);
            return {
                id: `${this.getMarkId()}-text-${key}`,
                type: "text",
                from: {
                    data: this.getDataIdOfFiltered()
                },
                layout: {
                    position: "content",
                    skipBeforeLayouted: !0
                },
                key: "flattenIndex",
                dependency: this.viewSpec.scales.map((scale => scale.id)).concat(SIGNAL_VIEW_BOX),
                animation: this.convertMarkAnimation(),
                encode: {
                    enter: Object.assign({
                        textAlign: "center",
                        textBaseline: "middle"
                    }, label[key].textStyle),
                    update: (datum, el, params) => ({
                        x: datum.labelRect ? (datum.labelRect.x0 + datum.labelRect.x1) / 2 : (datum.x0 + datum.x1) / 2,
                        y: datum.labelRect ? (datum.labelRect.y0 + datum.labelRect.y1) / 2 : (datum.y0 + datum.y1) / 2,
                        text: textGetter(datum.datum[datum.datum.length - 1])
                    })
                }
            };
        })) : null;
    }
}

TreemapSemanticMark.type = PlotMakType.treemap;
//# sourceMappingURL=treemap.js.map